#include "pch.h"
#include "UserLic.h"
#include "MtConverter.h"


USER_LIC::USER_LIC()
{
	uid = 0;
	expired = -1;
	log_count = 0;
	ea_count = 0;
	std::memset(logins, 0, sizeof(long) * COUNTACC);
}

USER_LIC::USER_LIC(USER_LIC&& ul) : uid{ ul.uid }, expired{ ul.expired }, log_count{ ul.log_count }, \
									ea_count{ ul.ea_count }, pEa{ std::move(ul.pEa) }
{
	memcpy(logins, ul.logins, sizeof(long) * COUNTACC);
}

USER_LIC::USER_LIC(const byte* pbyte)
{
	rsize_t sz = sizeof(int64_t) + sizeof(std::time_t) + sizeof(long) + sizeof(int64_t) * COUNTACC + sizeof(long);
	memcpy_s((byte* )this, sz, pbyte, sz);
	byte* s = (byte*)pbyte + sz;
	if (ea_count > 0) {
		int c = ea_count;
		for (int i = 0; i < c; i++) {
			EAUSER ea;
			memcpy_s((byte*)&ea, sizeof(EAUSER), s, sizeof(EAUSER) );
			AddEA(ea);
			s += sizeof(EAUSER);
		}
		ea_count = c;
	}
}


std::string USER_LIC::GetTimeExpired()
{
	return MtConverter::ToString(expired);
}

std::string USER_LIC::ToString()
{
	std::stringstream  ot;
	ot << "User ID: " << uid << "\r\n" << "Expired: " << GetTimeExpired() << "\r\n";
	ot << "Accounts (" << log_count << ") :\r\n";
	for (int i = 0; i < log_count; i++) 
		ot << " " << i << " Account: " << logins[i] << "\r\n";
	ot << "EA count (" << ea_count << ") :\r\n";
	for (size_t i = 0; i < ea_count; i++) 
		ot << " " << i << " " <<(*pEa[i]).ToString() << "\r\n";
	ot.flush();
	return ot.str();
}

size_t USER_LIC::ToArray(byte* pbyte)
{
	rsize_t sz = sizeof(int64_t) + sizeof(std::time_t) + sizeof(long) + sizeof(int64_t) * COUNTACC + sizeof(long);
	try {
		memcpy_s(pbyte, sz, (byte*)this, sz);
		byte* s = (byte*)pbyte + sz;
		if (ea_count > 0) {
			int c = ea_count;
			for (int i = 0; i < ea_count; i++) {
				pEa[i]->ToArray(s);
				s  += sizeof(EAUSER);
				sz += sizeof(EAUSER);
			}
		}
	} catch (std::exception& e) {
		std::cout << __FUNCTION__ << " " << e.what() << '\n';
		throw;
	}
	return sz;
}

void USER_LIC::AddEA(EA_USER eau)
{
	std::unique_ptr<EAUSER> pea = std::make_unique<EAUSER>(eau);
	pEa.emplace_back(std::move(pea) );
	ea_count++;
}

bool USER_LIC::AddAcc(long newAcc)
{
	if (log_count >= COUNTACC) return false;
	logins[log_count++] = newAcc;
	return false;
}

size_t USER_LIC::GetSize()
{
	rsize_t sz = sizeof(int64_t) + sizeof(std::time_t) + sizeof(long) + sizeof(int64_t) * COUNTACC + sizeof(long);
	for (int i = 0; i < ea_count; i++) sz += sizeof(EAUSER);
	return sz;
}

std::ostream& operator<< (std::ostream& out, USER_LIC& ul)
{
	try {
		out.write((const char*)&ul, sizeof(int64_t) + sizeof(std::time_t) + sizeof(long) + sizeof(int64_t) * COUNTACC + sizeof(long));
		for (auto i = ul.pEa.cbegin(); i != ul.pEa.cend(); i++)
			out << **i;
	} catch (std::exception& e) {
		std::cout << __FUNCTION__ << " " << e.what() << '\n';
		throw;
	}
	return out;
}

std::istream& operator>> (std::istream& in, USER_LIC& ul) {
	try {
		in.read((char* )&ul, sizeof(int64_t) + sizeof(std::time_t) + sizeof(long) + sizeof(int64_t) * COUNTACC + sizeof(long));
		if (ul.ea_count > 0) {
			ul.pEa.clear();
			int c = ul.ea_count;
			for (int i = 0; i < c; i++) {
				EAUSER ea;
				in >> ea;
				ul.AddEA(ea);
			}
			ul.ea_count = c;
		}
	} catch (std::exception& e) {
		std::cout << __FUNCTION__ << " " << e.what() << '\n';
		throw;
	}
	return in;
}